import sys
from os import environ

import pg8000.dbapi

import pytest


@pytest.fixture(scope="class")
def db_kwargs():
    db_connect = {
        'user': 'postgres',
        'password': 'pw'
    }

    for kw, var, f in [
                ('host', 'PGHOST', str),
                ('password', 'PGPASSWORD', str),
                ('port', 'PGPORT', int)
            ]:
        try:
            db_connect[kw] = f(environ[var])
        except KeyError:
            pass

    return db_connect


@pytest.fixture
def con(request, db_kwargs):
    conn = pg8000.dbapi.connect(**db_kwargs)

    def fin():
        conn.rollback()
        try:
            conn.close()
        except pg8000.dbapi.InterfaceError:
            pass

    request.addfinalizer(fin)
    return conn


@pytest.fixture
def cursor(request, con):
    cursor = con.cursor()

    def fin():
        cursor.close()

    request.addfinalizer(fin)
    return cursor


@pytest.fixture
def is_java():
    return 'java' in sys.platform.lower()



import os
import time

import pytest


@pytest.fixture
def has_tzset():

    # Neither Windows nor Jython 2.5.3 have a time.tzset() so skip
    if hasattr(time, 'tzset'):
        os.environ['TZ'] = "UTC"
        time.tzset()
        return True
    return False


# DBAPI compatible interface tests
@pytest.fixture
def db_table(con, has_tzset):
    con.run("START TRANSACTION")
    con.run(
        "CREATE TEMPORARY TABLE t1 "
        "(f1 int primary key, f2 int not null, f3 varchar(50) null) "
        "ON COMMIT DROP")
    con.run(
        "INSERT INTO t1 (f1, f2, f3) VALUES (:v1, :v2, :v3)",
        v1=1, v2=1, v3=None)
    con.run(
        "INSERT INTO t1 (f1, f2, f3) VALUES (:v1, :v2, :v3)",
        v1=2, v2=10, v3=None)
    con.run(
        "INSERT INTO t1 (f1, f2, f3) VALUES (:v1, :v2, :v3)",
        v1=3, v2=100, v3=None)
    con.run(
        "INSERT INTO t1 (f1, f2, f3) VALUES (:v1, :v2, :v3)",
        v1=4, v2=1000, v3=None)
    con.run(
        "INSERT INTO t1 (f1, f2, f3) VALUES (:v1, :v2, :v3)",
        v1=5, v2=10000, v3=None)
    return con


def test_named(db_table):
    res = db_table.run("SELECT f1, f2, f3 FROM t1 WHERE f1 > :f1", f1=3)
    for row in res:
        f1, f2, f3 = row


def test_row_count(db_table):
    db_table.run("SELECT * FROM t1")

    assert 5 == db_table.row_count

    db_table.run("UPDATE t1 SET f3 = :v WHERE f2 > 101", v="Hello!")
    assert 2 == db_table.row_count

    db_table.run("DELETE FROM t1")
    assert 5 == db_table.row_count


def test_prepared_statement(con):
    con.run('PREPARE gen_series AS SELECT generate_series(1, 10);')
    con.run('EXECUTE gen_series')



import pg8000.dbapi

import pytest


def test_gss(db_kwargs):
    """ Called by GitHub Actions with auth method gss.
    """

    # Should raise an exception saying gss isn't supported
    with pytest.raises(
            pg8000.dbapi.InterfaceError,
            match="Authentication method 7 not supported by pg8000."):
        pg8000.dbapi.connect(**db_kwargs)



from io import BytesIO

import pytest


@pytest.fixture
def db_table(request, con):
    con.run("START TRANSACTION")
    con.run(
        "CREATE TEMPORARY TABLE t1 "
        "(f1 int primary key, f2 int not null, f3 varchar(50) null) "
        "on commit drop")
    return con


def test_copy_to_with_table(db_table):
    db_table.run("INSERT INTO t1 (f1, f2, f3) VALUES (:v, :v, :v)", v=1)
    db_table.run("INSERT INTO t1 (f1, f2, f3) VALUES (:v, :v, :v)", v=2)
    db_table.run("INSERT INTO t1 (f1, f2, f3) VALUES (:v, :v, :v)", v=3)

    stream = BytesIO()
    db_table.run("copy t1 to stdout", stream=stream)
    assert stream.getvalue() == b"1\t1\t1\n2\t2\t2\n3\t3\t3\n"
    assert db_table.row_count == 3


def test_copy_to_with_query(db_table):
    stream = BytesIO()
    db_table.run(
        "COPY (SELECT 1 as One, 2 as Two) TO STDOUT WITH DELIMITER "
        "'X' CSV HEADER QUOTE AS 'Y' FORCE QUOTE Two", stream=stream)
    assert stream.getvalue() == b'oneXtwo\n1XY2Y\n'
    assert db_table.row_count == 1


def test_copy_from_with_table(db_table):
    stream = BytesIO(b"1\t1\t1\n2\t2\t2\n3\t3\t3\n")
    db_table.run("copy t1 from STDIN", stream=stream)
    assert db_table.row_count == 3

    retval = db_table.run("SELECT * FROM t1 ORDER BY f1")
    assert retval == [[1, 1, '1'], [2, 2, '2'], [3, 3, '3']]


def test_copy_from_with_query(db_table):
    stream = BytesIO(b"f1Xf2\n1XY1Y\n")
    db_table.run(
        "COPY t1 (f1, f2) FROM STDIN WITH DELIMITER 'X' CSV HEADER "
        "QUOTE AS 'Y' FORCE NOT NULL f1", stream=stream)
    assert db_table.row_count == 1

    retval = db_table.run("SELECT * FROM t1 ORDER BY f1")
    assert retval == [[1, 1, None]]


def test_copy_from_with_error(db_table):
    stream = BytesIO(b"f1Xf2\n\n1XY1Y\n")
    with pytest.raises(BaseException) as e:
        db_table.run(
            "COPY t1 (f1, f2) FROM STDIN WITH DELIMITER 'X' CSV HEADER "
            "QUOTE AS 'Y' FORCE NOT NULL f1", stream=stream)

    arg = {
        'S': ('ERROR',),
        'C': ('22P02',),
        'M': (
            'invalid input syntax for type integer: ""',
            'invalid input syntax for integer: ""'
        ),
        'W': ('COPY t1, line 2, column f1: ""',),
        'F': ('numutils.c',),
        'R': ('pg_atoi', 'pg_strtoint32')
    }
    earg = e.value.args[0]
    for k, v in arg.items():
        assert earg[k] in v



import socket
import struct

import pg8000

import pytest


def testUnixSocketMissing():
    conn_params = {
        'unix_sock': "/file-does-not-exist",
        'user': "doesn't-matter"
    }

    with pytest.raises(pg8000.dbapi.InterfaceError):
        pg8000.dbapi.connect(**conn_params)


def test_internet_socket_connection_refused():
    conn_params = {
        'port': 0,
        'user': "doesn't-matter"
    }

    with pytest.raises(
            pg8000.dbapi.InterfaceError,
            match="Can't create a connection to host localhost and port 0 "
            "\\(timeout is None and source_address is None\\)."):
        pg8000.dbapi.connect(**conn_params)


def testDatabaseMissing(db_kwargs):
    db_kwargs["database"] = "missing-db"
    with pytest.raises(pg8000.dbapi.DatabaseError):
        pg8000.dbapi.connect(**db_kwargs)


def test_notify(con):
    cursor = con.cursor()
    cursor.execute("select pg_backend_pid()")
    backend_pid = cursor.fetchall()[0][0]
    assert list(con.notifications) == []
    cursor.execute("LISTEN test")
    cursor.execute("NOTIFY test")
    con.commit()

    cursor.execute("VALUES (1, 2), (3, 4), (5, 6)")
    assert len(con.notifications) == 1
    assert con.notifications[0] == (backend_pid, "test", '')


def test_notify_with_payload(con):
    cursor = con.cursor()
    cursor.execute("select pg_backend_pid()")
    backend_pid = cursor.fetchall()[0][0]
    assert list(con.notifications) == []
    cursor.execute("LISTEN test")
    cursor.execute("NOTIFY test, 'Parnham'")
    con.commit()

    cursor.execute("VALUES (1, 2), (3, 4), (5, 6)")
    assert len(con.notifications) == 1
    assert con.notifications[0] == (backend_pid, "test", 'Parnham')


def testUnicodeDatabaseName(db_kwargs):
    db_kwargs["database"] = "pg8000_sn\uFF6Fw"

    # Should only raise an exception saying db doesn't exist
    with pytest.raises(pg8000.dbapi.DatabaseError, match='3D000'):
        pg8000.dbapi.connect(**db_kwargs)


def testBytesDatabaseName(db_kwargs):
    """ Should only raise an exception saying db doesn't exist """

    db_kwargs["database"] = bytes("pg8000_sn\uFF6Fw", 'utf8')
    with pytest.raises(pg8000.dbapi.DatabaseError, match='3D000'):
        pg8000.dbapi.connect(**db_kwargs)


def testBytesPassword(con, db_kwargs):
    # Create user
    username = 'boltzmann'
    password = 'cha\uFF6Fs'
    cur = con.cursor()
    cur.execute(
        "create user " + username + " with password '" + password + "';")
    con.commit()

    db_kwargs['user'] = username
    db_kwargs['password'] = password.encode('utf8')
    db_kwargs['database'] = 'pg8000_md5'
    with pytest.raises(pg8000.dbapi.DatabaseError, match='3D000'):
        pg8000.dbapi.connect(**db_kwargs)

    cur.execute("drop role " + username)
    con.commit()


def test_broken_pipe(con, db_kwargs):
    with pg8000.dbapi.connect(**db_kwargs) as db1:
        cur1 = db1.cursor()
        cur2 = con.cursor()
        cur1.execute("select pg_backend_pid()")
        pid1 = cur1.fetchone()[0]

        cur2.execute("select pg_terminate_backend(%s)", (pid1,))
        try:
            cur1.execute("select 1")
        except Exception as e:
            assert isinstance(e, (socket.error, struct.error))


def testApplicatioName(db_kwargs):
    app_name = 'my test application name'
    db_kwargs['application_name'] = app_name
    with pg8000.dbapi.connect(**db_kwargs) as db:
        cur = db.cursor()
        cur.execute(
            'select application_name from pg_stat_activity '
            ' where pid = pg_backend_pid()')

        application_name = cur.fetchone()[0]
        assert application_name == app_name


def test_application_name_integer(db_kwargs):
    db_kwargs['application_name'] = 1
    with pytest.raises(
            pg8000.dbapi.InterfaceError,
            match="The parameter application_name can't be of type "
            "<class 'int'>."):
        pg8000.dbapi.connect(**db_kwargs)


def test_application_name_bytearray(db_kwargs):
    db_kwargs['application_name'] = bytearray(b'Philby')
    pg8000.dbapi.connect(**db_kwargs)
